@@ -27,6 +27,8 @@
 #include "include/utils/SkPaintFilterCanvas.h"
 #include "src/core/SkCanvasPriv.h"
 #include "src/core/SkRectPriv.h"
+#include "src/core/SkStringUtils.h"
+#include "src/gpu/ganesh/GrCanvas.h"
 #include "src/gpu/ganesh/GrRecordingContextPriv.h"
 #include "src/gpu/ganesh/GrRenderTargetProxy.h"
 #include "src/gpu/ganesh/GrSurfaceProxy.h"
@@ -44,10 +46,12 @@ class SkRegion;
 class UrlDataManager;
 struct SkDrawShadowRec;
 
-#if SK_GPU_V1
+#if defined(SK_GANESH)
 #include "src/gpu/ganesh/GrAuditTrail.h"
 #endif
 
+using namespace skia_private;
+
 #define SKDEBUGCANVAS_VERSION 1
 #define SKDEBUGCANVAS_ATTRIBUTE_VERSION "version"
 #define SKDEBUGCANVAS_ATTRIBUTE_COMMANDS "commands"
@@ -163,7 +167,7 @@ void DebugCanvas::drawTo(SkCanvas* originalCanvas, int index, int m) {
     DebugPaintFilterCanvas filterCanvas(originalCanvas);
     SkCanvas* finalCanvas = fOverdrawViz ? &filterCanvas : originalCanvas;
 
-#if SK_GPU_V1
+#if defined(SK_GANESH)
     auto dContext = GrAsDirectContext(finalCanvas->recordingContext());
 
     // If we have a GPU backend we can also visualize the op information
@@ -175,7 +179,7 @@ void DebugCanvas::drawTo(SkCanvas* originalCanvas, int index, int m) {
 #endif
 
     for (int i = 0; i <= index; i++) {
-#if SK_GPU_V1
+#if defined(SK_GANESH)
         GrAuditTrail::AutoCollectOps* acb = nullptr;
         if (at) {
             // We need to flush any pending operations, or they might combine with commands below.
@@ -190,7 +194,7 @@ void DebugCanvas::drawTo(SkCanvas* originalCanvas, int index, int m) {
         if (fCommandVector[i]->isVisible()) {
             fCommandVector[i]->execute(finalCanvas);
         }
-#if SK_GPU_V1
+#if defined(SK_GANESH)
         if (at && acb) {
             delete acb;
         }
@@ -224,7 +228,7 @@ void DebugCanvas::drawTo(SkCanvas* originalCanvas, int index, int m) {
         finalCanvas->drawRect(fAndroidClip, androidClipPaint);
     }
 
-#if SK_GPU_V1
+#if defined(SK_GANESH)
     // draw any ops if required and issue a full reset onto GrAuditTrail
     if (at) {
         // just in case there is global reordering, we flush the canvas before querying
@@ -241,11 +245,11 @@ void DebugCanvas::drawTo(SkCanvas* originalCanvas, int index, int m) {
 
         // get the render target of the top device (from the original canvas) so we can ignore ops
         // drawn offscreen
-        GrRenderTargetProxy* rtp = SkCanvasPriv::TopDeviceTargetProxy(originalCanvas);
+        GrRenderTargetProxy* rtp = skgpu::ganesh::TopDeviceTargetProxy(originalCanvas);
         GrSurfaceProxy::UniqueID proxyID = rtp->uniqueID();
 
         // get the bounding boxes to draw
-        SkTArray<GrAuditTrail::OpInfo> childrenBounds;
+        TArray<GrAuditTrail::OpInfo> childrenBounds;
         if (m == -1) {
             at->getBoundsByClientID(&childrenBounds, index);
         } else {
@@ -292,7 +296,7 @@ DrawCommand* DebugCanvas::getDrawCommandAt(int index) const {
     return fCommandVector[index];
 }
 
-#if SK_GPU_V1
+#if defined(SK_GANESH)
 GrAuditTrail* DebugCanvas::getAuditTrail(SkCanvas* canvas) {
     GrAuditTrail* at  = nullptr;
     auto ctx = canvas->recordingContext();
@@ -330,12 +334,12 @@ void DebugCanvas::cleanupAuditTrail(GrAuditTrail* at) {
         at->fullReset();
     }
 }
-#endif // SK_GPU_V1
+#endif // defined(SK_GANESH)
 
 void DebugCanvas::toJSON(SkJSONWriter&   writer,
                          UrlDataManager& urlDataManager,
                          SkCanvas*       canvas) {
-#if SK_GPU_V1
+#if defined(SK_GANESH)
     this->drawAndCollectOps(canvas);
 
     // now collect json
@@ -348,7 +352,7 @@ void DebugCanvas::toJSON(SkJSONWriter&   writer,
         writer.beginObject();  // command
         this->getDrawCommandAt(i)->toJSON(writer, urlDataManager);
 
-#if SK_GPU_V1
+#if defined(SK_GANESH)
         if (at) {
             writer.appendName(SKDEBUGCANVAS_ATTRIBUTE_AUDITTRAIL);
             at->toJson(writer, i);
@@ -358,13 +362,13 @@ void DebugCanvas::toJSON(SkJSONWriter&   writer,
     }
 
     writer.endArray();  // commands
-#if SK_GPU_V1
+#if defined(SK_GANESH)
     this->cleanupAuditTrail(at);
 #endif
 }
 
 void DebugCanvas::toJSONOpsTask(SkJSONWriter& writer, SkCanvas* canvas) {
-#if SK_GPU_V1
+#if defined(SK_GANESH)
     this->drawAndCollectOps(canvas);
 
     GrAuditTrail* at = this->getAuditTrail(canvas);
@@ -422,7 +426,7 @@ void DebugCanvas::didTranslate(SkScalar x, SkScalar y) {
 void DebugCanvas::onDrawAnnotation(const SkRect& rect, const char key[], SkData* value) {
     // Parse layer-releated annotations added in SkiaPipeline.cpp and RenderNodeDrawable.cpp
     // the format of the annotations is <Indicator|RenderNodeId>
-    SkTArray<SkString> tokens;
+    TArray<SkString> tokens;
     SkStrSplit(key, "|", kStrict_SkStrSplitMode, &tokens);
     if (tokens.size() == 2) {
         if (tokens[0].equals(kOffscreenLayerDraw)) {
